package info.textgrid.middleware;

import java.io.IOException;
import java.text.ParseException;
import java.util.Hashtable;
import java.util.Map;
import java.util.logging.Logger;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.json.JSONObject;
import info.textgrid.middleware.oaipmh.ListIdentifiersType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

/**
 * @author Max Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2022-09-23
 */
public class IdentifierListDelivererIDIOM extends IdentifierListDelivererAbstract {

  // **
  // STATICS
  // **

  private static Logger log = Logger.getLogger(IdentifierListDelivererIDIOM.class.getName());

  private static Map<String, Integer> cursorCollector = new Hashtable<String, Integer>();

  private RecordDelivererIDIOM idiomRecord;

  // **
  // CLASS
  // **

  // Set default to 30, can be changed in oaipmh.properties.
  private int idiomResponseSize = 30;

  /**
   * @param textgrid
   * @param dariah
   */
  public IdentifierListDelivererIDIOM(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /**
   *
   */
  @Override
  public ListIdentifiersType processIdentifierList(final String from, final String to,
      final String set, final String resumptionToken) throws ParseException, IOException {

    ListIdentifiersType identifierList = new ListIdentifiersType();
    BoolQueryBuilder recordFilterForClassicMayan;
    RangeQueryBuilder rangeQuery = QueryBuilders.rangeQuery("lastModified").from(from).to(to);

    BoolQueryBuilder projectQuery = QueryBuilders.boolQuery().must(
        QueryBuilders.matchPhraseQuery("project.id", "TGPR-0e926f53-1aba-d415-ecf6-539edcd8a318"));
    BoolQueryBuilder artefactQuery = QueryBuilders.boolQuery()
        .must(QueryBuilders.matchPhraseQuery("format", "text/tg.inputform+rdf+xml"))
        .must(QueryBuilders.matchPhraseQuery("notes", "ARTEFACT"));
    BoolQueryBuilder conedakorQuery = QueryBuilders.boolQuery()
        .must(QueryBuilders.matchPhraseQuery("format", "application/json"))
        .must(QueryBuilders.matchPhraseQuery("notes", "ConedaKorMediumData"));

    BoolQueryBuilder both = QueryBuilders.boolQuery().should(artefactQuery).should(conedakorQuery);

    BoolQueryBuilder bla =
        QueryBuilders.boolQuery().must(rangeQuery).must(projectQuery.filter(both));
    recordFilterForClassicMayan = bla;

    SearchResponse scrollResp;

    SearchRequest searchRequest = new SearchRequest("textgrid-nonpublic");
    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

    scrollResp = null;

    searchSourceBuilder.query(recordFilterForClassicMayan);
    searchSourceBuilder.size(this.idiomResponseSize);
    searchRequest.source(searchSourceBuilder);

    log.fine("es elient/index: " + (this.oaiEsClient == null ? "null" : this.oaiEsClient) + "/"
        + (this.oaiEsClient == null ? "null" : this.oaiEsClient.getEsIndex()));

    if (resumptionToken != null) {
      SearchScrollRequest scrollRequest = new SearchScrollRequest(resumptionToken);
      scrollRequest.scroll(TimeValue.timeValueHours(24L));
      try {
        scrollResp = this.oaiEsClient.getEsClient().scroll(scrollRequest, RequestOptions.DEFAULT);
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    } else {
      // Use given resumption token here.
      searchRequest.source(searchSourceBuilder);
      searchRequest.scroll(TimeValue.timeValueHours(24L));
      try {
        scrollResp = this.oaiEsClient.getEsClient().search(searchRequest, RequestOptions.DEFAULT);
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }

    String scrollID = scrollResp.getScrollId();

    long completeListSize = scrollResp.getHits().totalHits;
    if (completeListSize > 0) {
      setFoundItems(true);
      int i = 0;

      for (SearchHit hit : scrollResp.getHits().getHits()) {
        i++;
        String textgridURI =
            OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), "textgridUri");

        // FIXME BTW: We only need ID and DATESTAMP for ListIdentifiers, WHY do we get every
        // complete RECORD??
        identifierList.getHeader()
            .add(this.idiomRecord.getRecordById(textgridURI).getRecord().getHeader());
      }

      // Check the need for a resumption token!
      ResumptionTokenType responseToken = OAIPMHUtilities.getResumptionToken(completeListSize,
          resumptionToken, cursorCollector, scrollID, this.idiomResponseSize, i);
      if (responseToken != null) {
        identifierList.setResumptionToken(responseToken);
      }
    } else {
      setFoundItems(false);
    }

    return identifierList;
  }

  /**
   * @return
   */
  public int getIdiomResponseSize() {
    return this.idiomResponseSize;
  }

  /**
   * @param idiomResponseSize
   */
  public void setIdiomResponseSize(int idiomResponseSize) {
    this.idiomResponseSize = idiomResponseSize;
  }

  /**
   * @return
   */
  public static Map<String, Integer> getCursorCollector() {
    return cursorCollector;
  }

  /**
   * @return
   */
  public RecordDelivererIDIOM getIdiomRecord() {
    return this.idiomRecord;
  }

  /**
   * @param idiomRecord
   */
  public void setIdiomRecord(RecordDelivererIDIOM idiomRecord) {
    this.idiomRecord = idiomRecord;
  }

}
