package info.textgrid.middleware;

import java.io.IOException;
import java.text.ParseException;
import java.util.List;
import java.util.logging.Logger;
import javax.xml.datatype.DatatypeConfigurationException;
import org.elasticsearch.common.Strings;
import org.json.JSONObject;
import org.springframework.stereotype.Component;
import info.textgrid.middleware.common.LTPUtils;
import info.textgrid.middleware.common.RDFConstants;
import info.textgrid.middleware.common.TextGridMimetypes;
import info.textgrid.middleware.oaipmh.ContributorType;
import info.textgrid.middleware.oaipmh.DateType;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.OpenaireDescriptionType;
import info.textgrid.middleware.oaipmh.RelatedIdentifierType;
import info.textgrid.middleware.oaipmh.RelationType;
import info.textgrid.middleware.oaipmh.Resource;
import info.textgrid.middleware.oaipmh.Resource.AlternateIdentifiers;
import info.textgrid.middleware.oaipmh.Resource.AlternateIdentifiers.AlternateIdentifier;
import info.textgrid.middleware.oaipmh.Resource.Contributors;
import info.textgrid.middleware.oaipmh.Resource.Contributors.Contributor;
import info.textgrid.middleware.oaipmh.Resource.Creators;
import info.textgrid.middleware.oaipmh.Resource.Creators.Creator;
import info.textgrid.middleware.oaipmh.Resource.Creators.Creator.NameIdentifier;
import info.textgrid.middleware.oaipmh.Resource.Dates;
import info.textgrid.middleware.oaipmh.Resource.Dates.Date;
import info.textgrid.middleware.oaipmh.Resource.Descriptions;
import info.textgrid.middleware.oaipmh.Resource.Descriptions.Description;
import info.textgrid.middleware.oaipmh.Resource.Formats;
import info.textgrid.middleware.oaipmh.Resource.GeoLocations;
import info.textgrid.middleware.oaipmh.Resource.GeoLocations.GeoLocation;
import info.textgrid.middleware.oaipmh.Resource.Identifier;
import info.textgrid.middleware.oaipmh.Resource.RelatedIdentifiers;
import info.textgrid.middleware.oaipmh.Resource.RelatedIdentifiers.RelatedIdentifier;
import info.textgrid.middleware.oaipmh.Resource.ResourceType;
import info.textgrid.middleware.oaipmh.Resource.RightsList;
import info.textgrid.middleware.oaipmh.Resource.RightsList.Rights;
import info.textgrid.middleware.oaipmh.Resource.Sizes;
import info.textgrid.middleware.oaipmh.Resource.Subjects;
import info.textgrid.middleware.oaipmh.Resource.Subjects.Subject;
import info.textgrid.middleware.oaipmh.Resource.Titles;
import info.textgrid.middleware.oaipmh.Resource.Titles.Title;

/**
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2021-09-10
 * @since 2020-06-13
 */
@Component
public class RecordDelivererDATACITE extends RecordDelivererAbstract {

  private static Logger log = Logger.getLogger(RecordDelivererDATACITE.class.getName());

  // Container object for the result of the elasticsearch request. This object contains the source
  // of the response.
  private JSONObject jsonObj = new JSONObject();

  // This variables contains the field requested in the ElasticSearch Index. The content will be
  // given by a configuration file.
  private String oarIdentifierField;
  private String oarSizeField;
  private String[] oarTitleFields;
  private String[] oarDateFields;
  private String[] oarContributorFields;
  private String[] oarCreatorFields;
  private String[] oarLanguageFields;
  private String[] oarAlternateIdentifierFields;
  private String[] oarFormatFields;
  private String[] oarRightsFields;
  private String[] oarDescriptionFields;
  private String[] oarRelatedIdentifierFields;
  private String[] oarGeoLocationFields;
  private String[] oarVersionFields;
  private String[] oarSubjectFields;

  private String relationToWorkObject;
  private String handle;

  /**
   * @param textgrid flag to indicate that the content of the record is context of the TextGrid
   *        Repository.
   * @param dariah flag to indicate that the content of the record is context of the DARIAH
   *        Repository.
   * @throws IOException
   */
  public RecordDelivererDATACITE(boolean textgrid, boolean dariah) throws IOException {
    super(textgrid, dariah);
  }

  /**
   *
   */
  @Override
  public GetRecordType getRecordById(final String idInElasticSearchIndex)
      throws ParseException, DatatypeConfigurationException, IOException {

    String replacedID = idInElasticSearchIndex.replace(this.repositoryObjectURIPrefix, "");
    GetRecordType getRecordType = new GetRecordType();

    // Get ES object.
    this.jsonObj = new JSONObject(OAIPMHUtilities.getRcordByIDFromElasticSearch(this.oaiEsClient,
        replacedID, this.fields, Strings.EMPTY_ARRAY).getSource());

    if (this.jsonObj == null || this.jsonObj.isEmpty()) {
      throw new IOException(
          "No data could be retrieved from ElasticSearch for ID: " + idInElasticSearchIndex);
    }

    // Set response header.
    String datestamp = OAIPMHUtilities
        .datestampAsString(OAIPMHUtilities.fieldLoader(this.jsonObj, this.dateOfObjectCreation));

    String identifier = "";
    if (!idInElasticSearchIndex.startsWith(this.repositoryObjectURIPrefix)) {
      identifier = this.repositoryObjectURIPrefix + idInElasticSearchIndex;
    } else {
      identifier = idInElasticSearchIndex;
    }

    String setSpec = OAIPMHUtilities.fieldLoader(this.jsonObj, this.specField);
    String setSpecValue = OAIPMHUtilities.getSetSpec(setSpec, this.specFieldPrefix, identifier);

    HeaderType header =
        OAIPMHUtilities.computeResponseHeader(datestamp, identifier, setSpecValue);
    getRecordType = OAIPMHUtilities.getRecordType(this.setOpenAireRecord(), header);

    return getRecordType;
  }

  /**
   * Adding the openaire record fields to the openaire record object
   * 
   * @return
   * @throws ParseException
   * @throws DatatypeConfigurationException
   * @throws IOException
   */
  private Resource setOpenAireRecord()
      throws ParseException, DatatypeConfigurationException, IOException {

    Resource result = new Resource();

    result.setTitles(this.addTitles());
    result.setPublisher(addPublisher());
    result.setPublicationYear(this.addPublicationYear());
    result.setDates(this.addDates());
    result.setContributors(this.addContributors());
    result.setLanguage(this.addLanguae());
    result.setAlternateIdentifiers(this.addAlternateIdentifier());
    result.setFormats(this.addFormats());
    result.setRightsList(this.addRights());
    result.setDescriptions(this.addDescriptions());
    result.setRelatedIdentifiers(this.addRelatedIdentifiers());
    result.setIdentifier(this.addIdentifier());
    result.setCreators(this.addCreators());
    // FIXME Check if we can have empty elements here! Was: Check for null in this.addGeoLocation()
    // method!)
    result.setGeoLocations(this.addGeoLocation());
    result.setResourceType(addResourceType());
    result.setVersion(this.addVersion());
    // FIXME Check if we can have empty elements here! Was: Check for null in this.addSubjects()
    // method!)
    result.setSubjects(this.addSubjects());
    result.setSizes(this.addSize());

    return result;
  }

  /**
   * @return
   */
  private Sizes addSize() {
    Sizes sizes = new Sizes();

    String size = OAIPMHUtilities.fieldLoader(this.jsonObj, this.oarSizeField) + " Bytes";

    sizes.getSize().add(size);

    return sizes;
  }

  /**
   * @return
   */
  private Subjects addSubjects() {

    Subjects subjects = new Subjects();

    // **
    // TextGrid
    // **

    if (this.textgrid) {
      for (String subjectField : this.oarSubjectFields) {
        Subject subject = new Subject();
        subject.setValue(OAIPMHUtilities.fieldLoader(this.jsonObj, subjectField + ".value"));
        subject.setSchemeURI(OAIPMHUtilities.fieldLoader(this.jsonObj, subjectField + ".id.type"));
        subject.setSubjectScheme(
            OAIPMHUtilities.fieldLoader(this.jsonObj, subjectField + ".id.value"));
        subjects.getSubject().add(subject);
      }
    }

    // **
    // DARIAH
    // **

    else if (this.dariah) {
      for (String subjectField : this.oarSubjectFields) {
        List<String> sList = OAIPMHUtilities.fieldLoader(this.jsonObj, new String[] {subjectField});
        for (String s : sList) {
          Subject subject = new Subject();

          subject.setValue(s);
          // TODO Extract subject schema and schema URI somehow?
          // subject.setSchemeURI();
          // subject.setSubjectScheme();
          subjects.getSubject().add(subject);
        }
      }
    }

    return subjects;
  }

  /**
   * @return
   */
  private String addVersion() {

    String version = "";

    for (String versionField : this.oarVersionFields) {
      version = OAIPMHUtilities.fieldLoader(this.jsonObj, versionField);
    }

    return version;
  }

  /**
   * @return
   */
  private ResourceType addResourceType() {

    ResourceType resourceType = new ResourceType();

    // **
    // TextGrid
    // **

    if (this.textgrid) {
      for (String format : this.oarFormatFields) {
        String resourceValue = OAIPMHUtilities.fieldLoader(this.jsonObj, format);
        // TODO Set resource value here?
        resourceType.setValue(resourceValue);
        // Set collection for all aggregation types, take also into account: images and text.
        if (TextGridMimetypes.AGGREGATION_SET.contains(format)) {
          resourceType
              .setResourceTypeGeneral(info.textgrid.middleware.oaipmh.ResourceType.COLLECTION);
        } else if (TextGridMimetypes.IMAGE_SET.contains(format)) {
          resourceType
              .setResourceTypeGeneral(info.textgrid.middleware.oaipmh.ResourceType.IMAGE);
        } else if (TextGridMimetypes.ORIGINAL_SET.contains(format)) {
          resourceType
              .setResourceTypeGeneral(info.textgrid.middleware.oaipmh.ResourceType.TEXT);
        }
        // Use dataset for everything else.
        else {
          resourceType.setResourceTypeGeneral(info.textgrid.middleware.oaipmh.ResourceType.DATASET);
        }
        // We take only the first of all the resource types!
        if (resourceType.getValue() != null && !resourceType.getValue().isEmpty()) {
          break;
        }
      }
    }

    // **
    // DARIAH
    // **

    else if (this.dariah) {

      for (String format : this.oarFormatFields) {
        // Only the first value is taken here! We do not need to create a list!
        // TODO Please see addLanguage(), maybe do it as done there to use the other fielsLoader()
        // method!
        String resourceValue = OAIPMHUtilities.fieldLoader(this.jsonObj, format);

        resourceType.setValue(resourceValue);
        // Set collection for DARIAH collection type here, data object for everything else.
        if (TextGridMimetypes.DARIAH_COLLECTION.equals(format)) {
          resourceType
              .setResourceTypeGeneral(info.textgrid.middleware.oaipmh.ResourceType.COLLECTION);
        }
        // Use dataset for everything else.
        else {
          resourceType.setResourceTypeGeneral(info.textgrid.middleware.oaipmh.ResourceType.DATASET);
        }
        // We take only the first of all the resource types!
        if (resourceType.getValue() != null && !resourceType.getValue().isEmpty()) {
          break;
        }
      }
    }

    return resourceType;
  }

  /**
   * 
   */
  private GeoLocations addGeoLocation() {

    GeoLocations geoLocations = new GeoLocations();

    for (String geoLocationField : this.oarGeoLocationFields) {
      GeoLocation geoLocation = new GeoLocation();
      geoLocation.setGeoLocationPlace(
          OAIPMHUtilities.fieldLoader(this.jsonObj, geoLocationField + ".value"));
      geoLocations.getGeoLocation().add(geoLocation);
    }

    // if (geoLocations.getGeoLocation().get(0).getGeoLocationPlace() != null) {
    // this.resource.setGeoLocations(geoLocations);
    // }

    return geoLocations;
  }

  /**
   * @return
   * @throws IOException
   */
  private RelatedIdentifiers addRelatedIdentifiers() throws IOException {

    RelatedIdentifiers relatedIdentifiers = new RelatedIdentifiers();

    for (String relatedIdentifierField : this.oarRelatedIdentifierFields) {

      // **
      // TextGrid
      // *

      if (this.textgrid) {
        String[] workFields = {this.handle};
        RelatedIdentifier relatedIdentifier = new RelatedIdentifier();
        relatedIdentifier.setRelatedIdentifierType(RelatedIdentifierType.HANDLE);
        // relatedIdentifier.setRelatedMetadataScheme(TGConstants.TEXTGRID_METADATASCHEME);
        relatedIdentifier.setRelationType(RelationType.IS_PART_OF);
        // relatedIdentifier.setSchemeType("XSD");
        // relatedIdentifier.setSchemeURI(TGConstants.TEXTGRID_METADATASCHEME_URI);

        String uriForWork = OAIPMHUtilities.fieldLoader(this.jsonObj, relatedIdentifierField)
            .replace(this.repositoryObjectURIPrefix, "");

        relatedIdentifier
            .setValue(
                OAIPMHUtilities
                    .fieldLoader(
                        new JSONObject(
                            OAIPMHUtilities.getRcordByIDFromElasticSearch(this.oaiEsClient,
                                uriForWork, workFields, Strings.EMPTY_ARRAY).getSource()),
                        this.handle));
        relatedIdentifiers.getRelatedIdentifier().add(relatedIdentifier);
      }

      // **
      // DARIAH
      // **

      else if (this.dariah) {
        List<String> relatedIdentiferList =
            OAIPMHUtilities.fieldLoader(this.jsonObj, new String[] {relatedIdentifierField});
        for (String i : relatedIdentiferList) {

          RelatedIdentifier relatedID = new RelatedIdentifier();
          if (i.startsWith(RDFConstants.HDL_PREFIX)) {
            relatedID.setRelatedIdentifierType(RelatedIdentifierType.HANDLE);
            relatedID.setValue(LTPUtils.omitHdlPrefix(i));
          } else if (i.startsWith(RDFConstants.DOI_PREFIX)) {
            relatedID.setRelatedIdentifierType(RelatedIdentifierType.DOI);
            relatedID.setValue(LTPUtils.omitDoiPrefix(i));
          } else if (i.startsWith("http")) {
            relatedID.setRelatedIdentifierType(RelatedIdentifierType.URL);
          }
          // TODO Check other prefixes and add other values, too?
          else {
            // TODO No type OTHER existing! Value is mandatory, so we chose URL, what else can we
            // do?
            relatedID.setRelatedIdentifierType(RelatedIdentifierType.URL);
            relatedID.setValue(i);
          }
          // Relation type is REFERENCES for the time being (coming from dc:relation at the moment).
          relatedID.setRelationType(RelationType.REFERENCES);
          relatedIdentifiers.getRelatedIdentifier().add(relatedID);
        }
      }
    }

    return relatedIdentifiers;
  }

  /**
   * @return
   * @throws IOException
   */
  private Descriptions addDescriptions() throws IOException {

    Descriptions descriptions = new Descriptions();

    for (String descriptionField : this.oarDescriptionFields) {

      // **
      // TextGrid
      // **

      if (this.textgrid) {

        String idForWorkObject =
            OAIPMHUtilities.fieldLoader(this.jsonObj, this.relationToWorkObject);
        if (idForWorkObject != null) {
          idForWorkObject = idForWorkObject.replace(this.repositoryObjectURIPrefix, "");
          JSONObject resultOfFurtherObject =
              new JSONObject(OAIPMHUtilities.getRcordByIDFromElasticSearch(this.oaiEsClient,
                  idForWorkObject, this.workFields, Strings.EMPTY_ARRAY).getSource());
          // Create abstract for every dc:description.
          List<String> abstractsFromWork =
              OAIPMHUtilities.fieldLoader(resultOfFurtherObject, new String[] {descriptionField});
          for (String d : abstractsFromWork) {
            Description description = new Description();
            description.setDescriptionType(OpenaireDescriptionType.ABSTRACT);
            description.getContent().add(d);
            descriptions.getDescription().add(description);
          }
        }
      }

      // **
      // DARIAH
      // **

      else if (this.dariah) {
        // Here the version of fieldLoader() is used, that delivers a LIST of strings!
        // TODO Check if you can adapt the other method to only have ONE method left!
        List<String> descriptionsFromMetadata =
            OAIPMHUtilities.fieldLoader(this.jsonObj, new String[] {descriptionField});

        // Create abstract for every dc:description.
        for (String d : descriptionsFromMetadata) {
          Description description = new Description();
          description.setDescriptionType(OpenaireDescriptionType.ABSTRACT);
          description.getContent().add(d);
          descriptions.getDescription().add(description);
        }
      }
    }

    return descriptions;
  }

  /**
   * @return
   */
  private RightsList addRights() {

    RightsList rightsList = new RightsList();

    // **
    // Textgrid
    // **

    if (this.textgrid) {
      for (String rightsField : this.oarRightsFields) {
        Rights rights = new Rights();
        rights.setRightsURI(OAIPMHUtilities.fieldLoader(this.jsonObj, rightsField + ".licenseUri"));
        rights.setValue(OAIPMHUtilities.fieldLoader(this.jsonObj, rightsField + ".value"));
        rightsList.getRights().add(rights);
      }
      if (rightsList.getRights().get(0).getValue() != null) {
        Rights openAccesRight = new Rights();
        openAccesRight.setRightsURI("info:eu-repo/semantics/openAccess");
        rightsList.getRights().add(openAccesRight);
      }
    }

    // **
    // DARIAH
    // **

    else if (this.dariah) {
      for (String rightsField : this.oarRightsFields) {
        Rights rights = new Rights();

        List<String> rList = OAIPMHUtilities.fieldLoader(this.jsonObj, new String[] {rightsField});

        for (String r : rList) {

          // Set rights URI assuming every "http://" or "https://" really IS an URI.
          if (r.startsWith("http")) {
            rights.setRightsURI(r);
          }
          // Set value in every case, even if doubled with URI.
          rights.setValue(r);
        }

        rightsList.getRights().add(rights);
      }
    }

    return rightsList;
  }

  /**
   * @return
   */
  private Formats addFormats() {

    Formats formats = new Formats();

    for (String formatField : this.oarFormatFields) {
      formats.getFormat().add(OAIPMHUtilities.fieldLoader(this.jsonObj, formatField));
    }

    return formats;
  }

  /**
   * @return
   */
  private AlternateIdentifiers addAlternateIdentifier() {

    AlternateIdentifiers alternateIdentifiers = new AlternateIdentifiers();

    for (String alternateIdentifierField : this.oarAlternateIdentifierFields) {

      // **
      // TextGrid
      // **

      if (this.textgrid) {
        AlternateIdentifier alternateIdentifier = new AlternateIdentifier();
        alternateIdentifier
            .setValue(OAIPMHUtilities.fieldLoader(this.jsonObj, alternateIdentifierField));
        alternateIdentifier.setAlternateIdentifierType(OAIPMHConstants.URI_IDENTIIER_TYPE);
        alternateIdentifiers.getAlternateIdentifier().add(alternateIdentifier);
      }

      // **
      // DARIAH
      // **

      else if (this.dariah) {
        List<String> alternateIdentifierList =
            OAIPMHUtilities.fieldLoader(this.jsonObj, new String[] {alternateIdentifierField});
        for (String i : alternateIdentifierList) {
          AlternateIdentifier alternateID = new AlternateIdentifier();
          if (i.startsWith(RDFConstants.HDL_PREFIX)) {
            alternateID.setAlternateIdentifierType("HANDLE");
            alternateID.setValue(LTPUtils.omitHdlPrefix(i));
          } else if (i.startsWith(RDFConstants.DOI_PREFIX)) {
            alternateID.setAlternateIdentifierType("DOI");
            alternateID.setValue(LTPUtils.omitDoiPrefix(i));
          } else {
            alternateID.setAlternateIdentifierType("OTHER");
            alternateID.setValue(i);
          }
          alternateIdentifiers.getAlternateIdentifier().add(alternateID);
        }
      }
    }

    return alternateIdentifiers;
  }

  /**
   * @return
   */
  private String addLanguae() {

    String language = null;

    // TODO Only ONE language is permitted in Datacite schema? What if we have more then one in
    // DARIAH DC data? Take only first one for the moment!
    for (String languageField : this.oarLanguageFields) {
      List<String> langs = OAIPMHUtilities.fieldLoader(this.jsonObj, new String[] {languageField});
      if (!langs.isEmpty()) {
        language = langs.get(0);
      }
    }

    return language;
  }

  /**
   * @return
   */
  private Contributors addContributors() {

    Contributors contributors = new Contributors();

    for (String contributorField : this.oarContributorFields) {

      // **
      // TextGrid
      // **

      if (this.textgrid) {

        Contributor contributorInOpenAireRecord = new Contributor();

        if (contributorField.equals("project")) {
          String fieldContent =
              OAIPMHUtilities.fieldLoader(this.jsonObj, contributorField + ".value");
          if (fieldContent != null && !fieldContent.isEmpty()) {
            contributorInOpenAireRecord.setContributorName(fieldContent);
            contributorInOpenAireRecord.setContributorType(ContributorType.OTHER);
            info.textgrid.middleware.oaipmh.Resource.Contributors.Contributor.NameIdentifier name =
                new info.textgrid.middleware.oaipmh.Resource.Contributors.Contributor.NameIdentifier();
            name.setNameIdentifierScheme("textgrid");
            name.setValue(OAIPMHUtilities.fieldLoader(this.jsonObj, contributorField + ".id"));
            name.setSchemeURI("https://textgridlab.org/schema/textgrid-metadata_2010.xsd");
            contributorInOpenAireRecord.setNameIdentifier(name);
            contributors.getContributor().add(contributorInOpenAireRecord);
          }
        } else {
          String fieldContent = OAIPMHUtilities.fieldLoader(this.jsonObj, contributorField);
          if (fieldContent != null && !fieldContent.isEmpty()) {
            contributorInOpenAireRecord.setContributorName(fieldContent);
            contributorInOpenAireRecord.setContributorType(ContributorType.DATA_MANAGER);
            contributors.getContributor().add(contributorInOpenAireRecord);
            /*
             * info.textgrid.middleware.oaipmh.Resource.Contributors.Contributor.NameIdentifier name
             * = new
             * info.textgrid.middleware.oaipmh.Resource.Contributors.Contributor.NameIdentifier();
             * name.setNameIdentifierScheme("textgrid"); name.setValue("");
             * name.setSchemeURI("http://www.textgridlab.org/schema/textgrid-metadata_2010.xsd");
             * contributorInOpenAireRecord.setNameIdentifier(name);
             */
          }
        }
      }

      // **
      // DARIAH
      // **

      else if (this.dariah) {

        List<String> contributorList =
            OAIPMHUtilities.fieldLoader(this.jsonObj, new String[] {contributorField});
        for (String i : contributorList) {
          Contributor contributorInOpenAireRecord = new Contributor();
          if (i != null && !i.isEmpty()) {
            contributorInOpenAireRecord.setContributorName(i);
            if (contributorField.equals("administrativeMetadata.dcterms:creator")) {
              contributorInOpenAireRecord.setContributorType(ContributorType.DATA_MANAGER);
            } else {
              contributorInOpenAireRecord.setContributorType(ContributorType.OTHER);
            }
            contributors.getContributor().add(contributorInOpenAireRecord);
          }
        }
      }
    }

    return contributors;
  }

  /**
   * @return
   */
  private Dates addDates() {

    Dates dates = new Dates();

    for (String dateField : this.oarDateFields) {

      Date dateInOpenAireRecord = new Date();

      // NOTE Date must be taken from oar date fields, not from dateOfObjectCration! That only must
      // be used in the record's header!
      // String date = OAIPMHUtilities.convertDateFormat(OAIPMHUtilities.fieldLoader(this.jsonObj,
      // this.dateOfObjectCreation).toString()).toXMLFormat();
      // dateInOpenAireRecord.setValue(date);

      try {
        // Only one value per field is needed here (I think).
        String dateValue = OAIPMHUtilities
            .convertDateFormat(OAIPMHUtilities.fieldLoader(this.jsonObj, dateField).toString())
            .toXMLFormat();
        dateInOpenAireRecord.setValue(dateValue);

        // TODO Use extra created, issued, and updated fields in configuration!

        // Filter TextGrid values for date type.
        if (dateField.equals("created")) {
          dateInOpenAireRecord.setDateType(DateType.CREATED);
        }
        if (dateField.equals("issued")) {
          dateInOpenAireRecord.setDateType(DateType.ISSUED);
        }
        if (dateField.equals("lastModified")) {
          dateInOpenAireRecord.setDateType(DateType.UPDATED);
        }

        // Filter DARIAH values for date type.
        if (dateField.equals("administrativeMetadata.dcterms:created")) {
          dateInOpenAireRecord.setDateType(DateType.CREATED);
        }
        if (dateField.equals("administrativeMetadata.dcterms:modified")) {
          dateInOpenAireRecord.setDateType(DateType.UPDATED);
        }
      } catch (ParseException | DatatypeConfigurationException e) {
        log.severe("could not parse date field: " + e.getMessage());
      }

      dates.getDate().add(dateInOpenAireRecord);
    }

    return dates;
  }

  /**
   * @return
   */
  private Identifier addIdentifier() {

    Identifier identifier = new Identifier();
    String idValue = OAIPMHUtilities.fieldLoader(this.jsonObj, this.oarIdentifierField);

    // **
    // TextGrid
    // **

    if (this.textgrid) {
      // We must set Handle here for TG, cause we do not have a DOI! So this is not valid due to
      // Datacite schema validation!
      // TODO Please check some time!
      identifier.setIdentifierType("Handle");
      identifier.setValue(LTPUtils.omitHdlPrefix(idValue));
    }

    // **
    // DARIAH
    // **

    else if (this.dariah) {
      // We set DOI for DH, 'cause we HAVE it! :-D
      identifier.setIdentifierType("DOI");
      identifier.setValue(LTPUtils.omitDoiPrefix(idValue));
    }

    return identifier;
  }

  /**
   * @return
   */
  private Creators addCreators() {

    Creators creators = new Creators();

    // **
    // TextGrid
    // **

    if (this.textgrid) {

      // TODO: Fields from configFile

      Creator creator = new Creator();
      creator.setCreatorName(OAIPMHUtilities.fieldLoader(this.jsonObj, "edition.agent.value"));
      NameIdentifier nameIdentifier = new NameIdentifier();
      // TODO: creatorID Field from config file
      String creatorID = OAIPMHUtilities.fieldLoader(this.jsonObj,
          "edition.source.bibliographicCitation.author.id");

      if (creatorID != null && creatorID.contains(":")) {
        nameIdentifier.setValue(creatorID.split(":")[1]);
        nameIdentifier.setNameIdentifierScheme(creatorID.split(":")[0]);
        // TODO Check if the scheme URI is correct
        nameIdentifier.setSchemeURI("https://ref.de.dariah.eu/pndsearch/pndquery.xql?id=");
      } else {
        nameIdentifier.setValue(creatorID);
      }
      creator.setNameIdentifier(nameIdentifier);
      nameIdentifier.setSchemeURI("https://ref.de.dariah.eu/pndsearch/pndquery.xql?id=");
      creators.getCreator().add(creator);
    }

    // **
    // DARIAH
    // **

    else if (this.dariah) {

      for (String cFields : this.oarCreatorFields) {

        List<String> creatorsList =
            OAIPMHUtilities.fieldLoader(this.jsonObj, this.oarCreatorFields);
        for (String c : creatorsList) {
          Creator creator = new Creator();
          creator.setCreatorName(c);
          creators.getCreator().add(creator);
        }
      }

    }

    return creators;
  }

  /**
   * @return
   */
  private Titles addTitles() {

    Titles titles = new Titles();

    for (String titleField : this.oarTitleFields) {

      List<String> titleList = OAIPMHUtilities.fieldLoader(this.jsonObj, this.oarTitleFields);
      for (String title : titleList) {
        Title tileInOpenAireRecord = new Title();
        tileInOpenAireRecord.setValue(title);
        // Title type OTHER is not permitted, leave type out for the moment!
        // tileInOpenAireRecord.setTitleType(TitleType.OTHER);
        titles.getTitle().add(tileInOpenAireRecord);
      }
    }

    return titles;
  }

  /**
   * @return
   */
  private String addPublisher() {

    String result = "";

    /**
     * Publisher fields
     */
    // TODO: fields from config file
    // TODO: for each field
    // TODO: Adapt to DARIAH!
    // return OAIPMHUtilities.fieldLoader(jsonObj,
    // "edition.source.bibliographicCitation.publisher.value");

    if (this.textgrid) {
      result = "TextGrid";
    } else if (this.dariah) {
      result = "DARIAH-DE";
    }

    return result;
  }

  /**
   * @return
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  private String addPublicationYear() throws ParseException, DatatypeConfigurationException {
    return Integer.toString(OAIPMHUtilities
        .convertDateFormat(OAIPMHUtilities.fieldLoader(this.jsonObj, this.dateOfObjectCreation))
        .getYear());
  }

  // **
  // GETTERS & SETTERS
  // **

  /**
   * @return
   */
  public String[] getOarTitleFields() {
    return this.oarTitleFields;
  }

  /**
   * @param oarTitleFields
   */
  public void setOarTitleFields(String[] oarTitleFields) {
    this.oarTitleFields = oarTitleFields;
  }

  /**
   * @return
   */
  public String getOarIdentifierField() {
    return this.oarIdentifierField;
  }

  /**
   * @param oarIdentifierField
   */
  public void setOarIdentifierField(String oarIdentifierField) {
    this.oarIdentifierField = oarIdentifierField;
  }

  /**
   * @return
   */
  public String[] getOarDateFields() {
    return this.oarDateFields;
  }

  /**
   * @param oarDateFields
   */
  public void setOarDateFields(String[] oarDateFields) {
    this.oarDateFields = oarDateFields;
  }

  /**
   * @return
   */
  public String[] getOarCreatorFields() {
    return this.oarCreatorFields;
  }

  /**
   * @param oarCreatorFields
   */
  public void setOarCreatorFields(String[] oarCreatorFields) {
    this.oarCreatorFields = oarCreatorFields;
  }

  /**
   * @return
   */
  public String[] getOarContributorFields() {
    return this.oarContributorFields;
  }

  /**
   * @param oarContributorFields
   */
  public void setOarContributorFields(String[] oarContributorFields) {
    this.oarContributorFields = oarContributorFields;
  }

  /**
   * @return
   */
  public String[] getOarLanguageFields() {
    return this.oarLanguageFields;
  }

  /**
   * @param oarLanguageFields
   */
  public void setOarLanguageFields(String[] oarLanguageFields) {
    this.oarLanguageFields = oarLanguageFields;
  }

  /**
   * @return
   */
  public String[] getOarAlternateIdentifierFields() {
    return this.oarAlternateIdentifierFields;
  }

  /**
   * @param oarAlternateIdentifierFields
   */
  public void setOarAlternateIdentifierFields(String[] oarAlternateIdentifierFields) {
    this.oarAlternateIdentifierFields = oarAlternateIdentifierFields;
  }

  /**
   * @return
   */
  public String[] getOarFormatFields() {
    return this.oarFormatFields;
  }

  /**
   * @param oarFormatFields
   */
  public void setOarFormatFields(String[] oarFormatFields) {
    this.oarFormatFields = oarFormatFields;
  }

  /**
   * @return
   */
  public String[] getOarRightsFields() {
    return this.oarRightsFields;
  }

  /**
   * @param oarRightsFields
   */
  public void setOarRightsFields(String[] oarRightsFields) {
    this.oarRightsFields = oarRightsFields;
  }

  /**
   * @return
   */
  public String[] getOarDescriptionFields() {
    return this.oarDescriptionFields;
  }

  /**
   * @param oarDescriptionFields
   */
  public void setOarDescriptionFields(String[] oarDescriptionFields) {
    this.oarDescriptionFields = oarDescriptionFields;
  }

  /**
   * @return
   */
  public String[] getOarRelatedIdentifierFields() {
    return this.oarRelatedIdentifierFields;
  }

  /**
   * @param oarRelatedIdentifierFields
   */
  public void setOarRelatedIdentifierFields(String[] oarRelatedIdentifierFields) {
    this.oarRelatedIdentifierFields = oarRelatedIdentifierFields;
  }

  /**
   * @return
   */
  public String getRelationToWorkObject() {
    return this.relationToWorkObject;
  }

  /**
   * @param relationToWorkObject
   */
  public void setRelationToWorkObject(String relationToWorkObject) {
    this.relationToWorkObject = relationToWorkObject;
  }

  /**
   * @return
   */
  public String[] getOarGeoLocationFields() {
    return this.oarGeoLocationFields;
  }

  /**
   * @param oarGeoLocationFields
   */
  public void setOarGeoLocationFields(String[] oarGeoLocationFields) {
    this.oarGeoLocationFields = oarGeoLocationFields;
  }

  /**
   * @return
   */
  public String getHandle() {
    return this.handle;
  }

  /**
   * @param handle
   */
  public void setHandle(String handle) {
    this.handle = handle;
  }

  /**
   * @return
   */
  public String[] getOarVersionFields() {
    return this.oarVersionFields;
  }

  /**
   * @param oarVersionFields
   */
  public void setOarVersionFields(String[] oarVersionFields) {
    this.oarVersionFields = oarVersionFields;
  }

  /**
   * @return
   */
  public String[] getOarSubjectFields() {
    return this.oarSubjectFields;
  }

  /**
   * @param oarSubjectFields
   */
  public void setOarSubjectFields(String[] oarSubjectFields) {
    this.oarSubjectFields = oarSubjectFields;
  }

  /**
   * @return
   */
  public String getOarSizeField() {
    return this.oarSizeField;
  }

  /**
   * @param oarSize
   */
  public void setOarSizeField(String oarSize) {
    this.oarSizeField = oarSize;
  }

}
