package info.textgrid.middleware;

import java.io.IOException;
import java.io.StringReader;
import java.text.ParseException;
import java.util.NoSuchElementException;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.classicmayan.tools.ClassicMayanMetsMods;
import org.elasticsearch.common.Strings;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.stereotype.Component;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.RecordType;

/**
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2022-09-27
 * @since 2019-03-12
 */
@Component
public class RecordDelivererIDIOM extends RecordDelivererAbstract {

  // **
  // FINALS
  // **

  private static final String TEXTGRID_URI_PREFIX = "textgrid:";

  // **
  // STATICS
  // **

  private static Logger log = Logger.getLogger(RecordDelivererIDIOM.class.getName());

  // **
  // CLASS
  // **

  private IDIOMImages idiomImages;

  private RecordType record = new RecordType();
  private String dateOfLastObjectModification;
  private String objectType;

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordDelivererIDIOM(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /**
   * <p>
   * Building the record XML object for the OAI-PMH response.
   * </p>
   * 
   * @throws ParseException
   * @throws IOException
   */
  @Override
  public GetRecordType getRecordById(final String id) throws ParseException, IOException {

    GetRecordType grt = new GetRecordType();

    log.fine("incoming ID: " + id);

    setDatestampsAndObjectType(id);

    // Get TG URI from ID.
    String textgridURI = id;
    if (!id.startsWith(TEXTGRID_URI_PREFIX)) {
      textgridURI = TEXTGRID_URI_PREFIX + id;
    }

    // Get base URI for ClassicMayanMetsMods and response header here.
    String textgridBaseURI = OAIPMHUtilities.getTextGridBaseURI(textgridURI);

    log.fine("textgrid uri/base uri: " + textgridURI + "/" + textgridBaseURI);
    log.fine("object type is: " + this.objectType);
    log.fine(this.objectType + " doc/dom: " + this.dateOfObjectCreation + "/"
        + this.dateOfLastObjectModification);

    if (this.objectType.equals("ARTEFACT")) {
      try {
        ClassicMayanMetsMods metsmods = new ClassicMayanMetsMods(
            textgridBaseURI,
            this.dateOfObjectCreation,
            this.dateOfLastObjectModification);

        log.fine("metsmods title: " + metsmods.getArtefactTitle());

        this.record.setMetadata(idiomMets(metsmods));

      } catch (JSONException | ParserConfigurationException | SAXException | IOException
          | ParseException | NoSuchElementException e) {
        log.severe("error! " + e.getClass().getName() + ": " + e.getMessage());
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }

    else if (this.objectType.startsWith("ConedaKorMediumData")) {

      log.fine("calling idiom image getRecordById()");

      GetRecordType idi = this.idiomImages.getRecordById(textgridURI);

      // Fixes #64
      if (idi != null) {
        this.record = idi.getRecord();
      }
    }

    // No setSpec needed here!
    // TODO Why not??
    String setSpec = "";
    HeaderType header =
        OAIPMHUtilities.computeResponseHeader(this.dateOfObjectCreation, textgridURI, setSpec);
    this.record.setHeader(header);

    grt.setRecord(this.record);

    return grt;
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * <p>
   * Getting the METS/MODS object for the ClassicMayan artifact and put them into metadata element
   * of the OAI-PMH response.
   * </p>
   * 
   * @param theMetsMods
   * @return
   * @throws ParserConfigurationException
   * @throws JSONException
   * @throws SAXException
   * @throws IOException
   * @throws ParseException
   */
  private static MetadataType idiomMets(ClassicMayanMetsMods theMetsMods)
      throws ParserConfigurationException, JSONException, SAXException, IOException,
      ParseException {

    MetadataType metadataMets = new MetadataType();
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder;

    builder = factory.newDocumentBuilder();
    Document doc = builder.parse(new InputSource(new StringReader(theMetsMods.getMets())));
    metadataMets.setAny(doc.getDocumentElement());

    return metadataMets;
  }

  /**
   * <p>
   * ElasticSearch request in non-public index to get the datestamps for the object creation and
   * last modification date in TextGridRep.
   * </p>
   * 
   * @param idInDatabase
   * @throws ParseException
   * @throws IOException
   */
  private void setDatestampsAndObjectType(final String idInDatabase)
      throws ParseException, IOException {

    String changedId = idInDatabase;

    // Remove prefix.
    if (idInDatabase.startsWith(TEXTGRID_URI_PREFIX)) {
      changedId = idInDatabase.substring(TEXTGRID_URI_PREFIX.length());
    }

    JSONObject json = new JSONObject();
    String[] searchFields = {TGConstants.CREATED, TGConstants.MODIFIED_FIELD, "notes"};

    json = new JSONObject(OAIPMHUtilities.getRcordByIDFromElasticSearch(this.oaiEsClient, changedId,
        searchFields, Strings.EMPTY_ARRAY).getSource());

    this.dateOfObjectCreation =
        OAIPMHUtilities.datestampAsString(OAIPMHUtilities.fieldLoader(json, TGConstants.CREATED));

    this.dateOfLastObjectModification = OAIPMHUtilities
        .datestampAsString(OAIPMHUtilities.fieldLoader(json, TGConstants.MODIFIED_FIELD));

    this.objectType = OAIPMHUtilities.fieldLoader(json, "notes");
  }

  // **
  // GETTERS & SETTERS
  // **

  /**
   * @return
   */
  public IDIOMImages getIdiomImages() {
    return this.idiomImages;
  }

  /**
   * @param idiomImages
   */
  public void setIdiomImages(IDIOMImages idiomImages) {
    this.idiomImages = idiomImages;
  }

}
